#pragma once

#include <unordered_map>
#include <boost/xpressive/xpressive.hpp>

namespace utl {

class HTMLEscape {
	std::unordered_map<std::wstring, wchar_t> map_;
	boost::xpressive::mark_tag tag_;
	boost::xpressive::mark_tag letter_;
	boost::xpressive::mark_tag digit_;
	boost::xpressive::mark_tag hex_;

public:
	HTMLEscape()
		: tag_(1)
		, letter_(2)
		, digit_(3)
		, hex_(4)
	{
		map_[L"quot"] = L'\"';
		map_[L"amp"] = L'&';
		map_[L"lt"] = L'<';
		map_[L"gt"] = L'>';
	}

	std::wstring operator()(const std::wstring& str)
	{
		try {
			return boost::xpressive::regex_replace(str, regex(), *this);
		} catch (...) {}
		return str;
	}

	boost::xpressive::wsregex regex() const
	{	
		using namespace boost::xpressive;

		try {
			const static wsregex rx = (L'<' >> (tag_ = +~as_xpr(L'>')) >> L'>')
				| (L'&' >> (letter_ = +range(L'a', L'z')) >> L';')
				| (L"&#"  >> (digit_ = +_d) >> L';')
				| icase(L"&#x"  >> (hex_ = +set[_d | range(L'a', L'f')]) >> L';');

			return rx;
		} catch (...) {}
		return wsregex();
	}

	std::wstring operator()(const boost::xpressive::wsmatch& what) const
	{
		using namespace boost::xpressive;

		std::wstring str;
		try {
			if (!what[tag_].str().empty())
			{
				const static wsregex rx = icase(skip(_s)(as_xpr(L"br")));
				if (regex_match(what[tag_].str(), rx))
				{
					str += L"\r\n";
					return str;
				}
			
				return str;
			}
			else if (!what[letter_].str().empty())
			{
				auto it = map_.find(what[letter_].str());
				if (it != map_.end())
				{
					str.push_back(it->second);
					return str;
				}
			}
			else if (!what[digit_].str().empty())
			{
				wchar_t c = (wchar_t)wcstol(what[digit_].str().c_str(), NULL, 10);
			
				str.push_back(c);
				return str;
			}
			else if (!what[hex_].str().empty())
			{
				wchar_t c = (wchar_t)wcstol(what[hex_].str().c_str(), NULL, 16);
			
				str.push_back(c);
				return str;
			}
			str = what[0];
		} catch (...) {}
		return str;
	}
};

class NameSplitter {
public:
	void operator()(const std::wstring& name, std::vector<std::pair<std::wstring, bool> >& dst)
	{
		using namespace boost::xpressive;
		try {
			dst.clear();
			HTMLEscape esc;
			for (wsregex_iterator it(name.begin(), name.end(), regex()), end; it != end; ++it)
			{
				dst.push_back(std::make_pair(esc(it->str(1)), it->str(2).empty()));
			}
		} catch (...) {}
	}

	boost::xpressive::wsregex regex() const
	{	
		using namespace boost::xpressive;
		try {
			const static wsregex rx = (s1 = -+_) >> icase((s2 = L"<b>") | L"</b>" | eos);
			return rx;
		} catch (...) {}
		return wsregex();
	}
};

}; // namespace utl