#pragma once

#define DEFAULT_USERAGENT	L"PCRPlayer/1.00"
#define DEFAULT_TIMEOUT		10000

#include <winhttp.h>

#pragma comment(lib, "winhttp.lib")

namespace network {


class URLParser {
private:
	bool valid_;
	std::wstring url_;
	std::wstring protocol_;
	std::wstring host_;
	std::wstring port_;
	std::wstring path_;
	std::wstring fullpath_;
	std::wstring file_;
	std::wstring query_;

public:
	URLParser();
	URLParser(const std::wstring& url);

	operator bool() { return valid_; }

	const std::wstring& url() { return url_; }
	const std::wstring& protocol() { return protocol_; }
	const std::wstring& host() { return host_; }
	//const std::wstring& port() { return port_; }
	WORD port();
	const std::wstring& path() { return path_; }
	const std::wstring& fullpath() { return fullpath_; }
	const std::wstring& file() { return file_; }
	const std::wstring& query() { return query_; }

private:
	const std::wstring::const_iterator parseProtocol(std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end);
	const std::wstring::const_iterator parseHost(std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end);
	const std::wstring::const_iterator parsePort(std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end);
	const std::wstring::const_iterator parsePath(std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end);
	const std::wstring::const_iterator parseQuery(std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end);
};

/*
class HttpUrl {
	bool valid_;
	std::wstring scheme_;
	std::wstring host_;
	std::wstring user_;
	std::wstring password_;
	std::wstring path_;
	std::wstring extra_;
	std::wstring file_;

	URL_COMPONENTS url_;

public:
	HttpUrl();
	HttpUrl(const std::wstring& url);
	virtual ~HttpUrl(){}

	operator bool() { return valid_; }

	std::wstring scheme();
	int schemeType();
	std::wstring host();
	WORD port();
	std::wstring user();
	std::wstring password();
	std::wstring path();
	std::wstring fullpath();
	std::wstring extra();
	std::wstring file();
};
*/

class HttpClient {
private:
	bool valid_;
	HINTERNET session_;
	HINTERNET connect_;
	HINTERNET request_;
	std::vector<char> buffer_;
	int timeout_;

public:
	HttpClient::HttpClient()
			: valid_(false)
			, session_(NULL)
			, connect_(NULL)
			, request_(NULL)
			, timeout_(10000)
	{}


	HttpClient::HttpClient(const std::wstring& url, const std::wstring& userAgent, const std::wstring& header, int timeout)
			: valid_(false)
			, session_(NULL)
			, connect_(NULL)
			, request_(NULL)
			, timeout_(timeout)
	{
		open(url, userAgent, header, timeout);
	}

	HttpClient::HttpClient(const std::wstring& url, const std::wstring& userAgent, int timeout)
			: valid_(false)
			, session_(NULL)
			, connect_(NULL)
			, request_(NULL)
			, timeout_(timeout)
	{
		open(url, userAgent, timeout);
	}

	virtual ~HttpClient();

	enum STATE {
		STATE_ERROR = -1,
		STATE_OK = 0,
		STATE_EOF = 1,
		STATE_TIMEOUT = 2,
	};

	operator bool() { return valid_; }

	bool open(const std::wstring& url, const std::wstring& userAgent, const std::wstring& header, int timeout);
	bool open(const std::wstring& url, const std::wstring& userAgent, int timeout)
	{
		return open(url, userAgent, std::wstring(), timeout);
	}

	bool post(const std::wstring& url, const std::wstring& userAgent, const char* data, DWORD size, const std::wstring& header, int timeout);
	bool post(const std::wstring& url, const std::wstring& userAgent, const char* data, DWORD size, int timeout)
	{
		return post(url, userAgent, data, size, std::wstring(), timeout);
	}

	bool post(const std::wstring& url, const std::wstring& userAgent, const std::vector<char>& data, const std::wstring& header, int timeout);
	bool post(const std::wstring& url, const std::wstring& userAgent, const std::vector<char>& data, int timeout)
	{
		return post(url, userAgent, data.data(), (DWORD)data.size(), std::wstring(), timeout);
	}

	bool post(const std::wstring& url, const std::wstring& userAgent, const std::string& data, const std::wstring& header, int timeout);
	bool post(const std::wstring& url, const std::wstring& userAgent, const std::string& data, int timeout)
	{
		return post(url, userAgent, data.data(), (DWORD)data.size(), std::wstring(), timeout);
	}

	int recv(std::vector<char>& dst);// 񕪂̕ԐM󂯎
	int get(std::vector<char>& dst);// t@C̍Ō܂Ŏ󂯎
	int read(void* dst, unsigned int size);// wTCY܂Ŏ󂯎

	bool queryStatus(DWORD& code);
	bool queryInfo(DWORD infoLevel, std::wstring& text);
	bool queryContentEncoding(std::wstring& text);
	bool queryLastModified(std::wstring& text);

	void close();

private:
	void closeHandle(HINTERNET& handle)
	{
		if (handle) { WinHttpCloseHandle(handle); handle = NULL; }
	}
};



} // namespace network

/*
class HttpClient {
	HINTERNET session_;
	HINTERNET connect_;
	HINTERNET request_;

	std::wstring proxy_;
	std::wstring proxyBypass_;

public:
	HttpClient();

	~HttpClient();

	void close();
	bool checkPlatform();
	bool open(const std::wstring& userAgent);
	bool open(const std::wstring& userAgent, const std::wstring& proxy);

	// session_
	bool version(DWORD major = 1, DWORD minor = 1);
	bool timeout(int resolve, int connect, int send, int receive);
	bool proxy(const std::wstring& domain_port);

	// session_
	bool connect(const std::wstring& domain, INTERNET_PORT port);
	bool connect(const std::wstring& domain, const std::wstring& port);
	bool connect(const std::wstring& domain_port);

	// connect_
	bool openRequest(const std::wstring& path);

	// request_
	bool addRequest(const std::wstring& header);
	bool sendRequest();

	bool receiveResponse(void);

	// request_
	bool queryInfo(DWORD infoLevel, std::wstring& text);
	bool queryStatus(DWORD& code);
	bool queryStatus(std::wstring& text);
	bool queryDate(SYSTEMTIME& time);
	bool queryDate(std::wstring& text);
	bool queryContentEncoding(std::wstring& text);

	bool readData(std::vector<char>& data);
};



class Network {
	HttpClient client_;

	std::wstring str_;

	std::wstring userAgent_;
	std::wstring proxy_;
	int timeout_;
	bool gzip_;

	bool gziped_;
	DWORD gzipSize_;

public:
	Network(
		const std::wstring& userAgent, int timeout)
		: userAgent_(userAgent)
		, proxy_(std::wstring())
		, timeout_(timeout)
		, gzip_(false)
		, gziped_(false)
		, gzipSize_(0)
	{}

	Network(
		const std::wstring& userAgent,
		const std::wstring& proxy,
		int timeout,
		bool gzip)
		: userAgent_(userAgent)
		, proxy_(proxy)
		, timeout_(timeout)
		, gzip_(gzip)
		, gziped_(false)
		, gzipSize_(0)
	{}

	bool connect(const std::wstring& url);
	bool request(const std::wstring& date = std::wstring());

	DWORD statusCode();
	std::wstring statusText();
	std::wstring date();

	bool get(const std::wstring& url);

	std::wstring str();
	const wchar_t* c_str();
	DWORD size();
	bool gziped();
	DWORD gzipSize();

private:
	bool gzipDecompress(std::vector<char>& dst, const std::vector<char>& src);
	std::wstring toUTF16(const std::string& src);
};

*/
